"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_get_1 = __importDefault(require("lodash.get"));
var lodash_has_1 = __importDefault(require("lodash.has"));
var opener = '{{';
var closer = '}}';
/**
 * @param template - The template string, containing zero or more placeholders.
 * @param data - The data structure providing values for interpolation.
 */
function interpolate(template, data) {
    var result = '';
    var index = 0;
    while (index < template.length) {
        var _a = findNextPlaceholder(template, index), text = _a.text, path = _a.path, newIndex = _a.index;
        result += text;
        if (path) {
            result += evaluatePlaceholder(path, data);
        }
        index = newIndex;
    }
    return result;
}
exports.default = interpolate;
function findNextPlaceholder(template, startIndex) {
    var openIndex = template.indexOf(opener, startIndex);
    if (openIndex !== -1) {
        var closeIndex = template.indexOf(closer, openIndex + opener.length);
        if (closeIndex === -1) {
            throw new Error('Unterminated placeholder');
        }
        var interveningText = template.slice(openIndex + opener.length, closeIndex);
        var path = extractPlaceholderPath(interveningText);
        return {
            text: template.slice(startIndex, openIndex),
            path: path,
            index: closeIndex + closer.length,
        };
    }
    else {
        return {
            text: template.slice(startIndex),
            path: undefined,
            index: template.length
        };
    }
}
function extractPlaceholderPath(placeholder) {
    var path = placeholder.trim();
    if (!path) {
        throw new Error('Empty placeholder');
    }
    if (!isValidPath(path)) {
        throw new Error(path + " is not a valid path");
    }
    return path;
}
function isValidPath(path) {
    return /^[a-zA-Z_$][a-zA-Z0-9_$]*(\.[a-zA-Z_$][a-zA-Z0-9_$]*)*$/.test(path);
}
function evaluatePlaceholder(path, data) {
    if (!lodash_has_1.default(data, path)) {
        throw new Error("Value " + path + " not found");
    }
    var value = lodash_get_1.default(data, path);
    if (typeof value !== 'string') {
        throw new Error('Values to be interpolated must be strings');
    }
    return value;
}
